# Web scraper looking for data on the web for a certain paper

import pandas as pd
from bs4 import BeautifulSoup
from EcoOpen.utils.keywords import keywords
import os
import requests
from pathlib import Path
from itertools import chain
import urllib3
from requests_html import HTMLSession
from tqdm import tqdm
import asyncio

from requests_html import HTMLSession, AsyncHTMLSession
from EcoOpen.utils.keywords import field_specific_repo, repositories

import validators
from pyDataverse.api import NativeApi, DataAccessApi
from pyDataverse.models import Dataverse

repos = field_specific_repo+repositories

test_urls = [
    'https://link.springer.com/article/10.1007/s10886-017-0919-8',
    "https://link.springer.com/article/10.1007/s10886-018-0942-4",
    "https://doi.org/10.1073/pnas.1211733109",
    "http://dx.doi.org/10.3955/046.091.0105",
    "https://doi.org/10.1016/j.scitotenv.2013.10.121",
    "https://doi.org/10.1111/nph.14333"
    ]

filetypes = [
    "csv", "xlsx", "xls", "txt", "pdf", "zip",
    "tar.gz", "tar", "gz", "json", "docx",
    "doc", "ods", "odt", "pptx", "ppt", "png",
    "jpg", "md", "exe", "tab"]

repositories = keywords["repositories"]

exclude = []
for i in filetypes:
    exclude.append(i.lower()+".")

def find_data_web(doi):
    
    url = "https://doi.org/" + doi
    #print(url)
    s = HTMLSession()
    r = s.get(url)
    # r.html.render()
    
    real_url = r.url
    
    domain = real_url.split('/')[2]
    soup = BeautifulSoup(r.html.html, 'lxml')
    supplementary_files = []
    closed_article_snippets = [
        "buy article",
        "access the full article",
        "purchase pdf",
    ]
    
    supplementary_snippets = [
        "supplementary",
        "supplemental",
        "additional",
        "appendix",
        "appendices",
        "suppl_file",
        "download"
    ]
    
    if any(snippet in str(soup).lower() for snippet in closed_article_snippets):
        # article is not publicly available
        pass
        # print("Article is not publicly available!")
        
    # print("SEARCHING FOR SUPPLEMENTARY MATERIAL")
    if any(snippet in str(soup).lower() for snippet in supplementary_snippets):
        links = soup.find_all('a')  
        for link in links:
            try:
                try:
                    if any(snippet in link.get('href').lower() for snippet in supplementary_snippets):
                        # print(link.get('href'))
                        supplementary_files.append(link.get('href'))
                        
                    if any(repo in link.get('href').lower() for repo in repos):
                        # print(link.get('href'))
                        supplementary_files.append(link.get('href'))
                except AttributeError:
                    pass
            except TypeError:
                pass
    
    # print("SEARCHING FOR DATA REPOSITORIES")
    for repo in repos:
        # print("searching for", repo)
        links = soup.find_all("a", href=True)
        for link in links:
            try:
                if repo in link.get('href').lower():
                    print(link.get('href'))
                    supplementary_files.append(link.get('href'))
            except TypeError:
                pass

    # if http not in link, add domain
    new_links = []
    for i in supplementary_files:
        if "http" not in i:
            i = "https://"+domain + i
            
            new_links.append(i)
        else:
            new_links.append(i)
    return new_links

    

def download_file(url, output_dir):
    response = requests.get(url)
    filename = url.split('/')[-1]
    if "?" in filename:
        filename = filename.split("?")[0]
    with open(os.path.join(output_dir, filename), 'wb') as f:
        f.write(response.content)


def download_osf_file(url, output_dir, file_name):
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
    }
    # print(url)
    # print(output_dir)
    # print(file_name)
    try:
        # Send a GET request to the URL
        response = requests.get(url,  headers=headers, stream=True)
        
        # Check if the request was successful
        if response.status_code == 200:
            # Create the output directory if it doesn't exist
            os.makedirs(output_dir, exist_ok=True)
            
            # Define the file path
            file_path = os.path.join(output_dir, file_name)
            
            # Save the content of the response as a file
            with open(file_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=8192):
                    file.write(chunk)
            
            # print(f"File downloaded successfully and saved as {file_path}")
            return file_path
        else:
            # print(f"Failed to download file. Status code: {response.status_code}")
            return None
    except requests.exceptions.RequestException:
        return None

def get_data_from_link(link, output_dir="examples/data"):
    # print(link, "!!")
    
    # There can be three types of links
    # 1. repository links
    # 2. site link
    # 3. direct download link
        
    #test if output_dir exists
    domain = link.split('/')[2]
    source = domain.replace("www.", "").replace(".", "_")
    for r in repositories:
        if r in link:
            source = r
            break
    output_dir = os.path.expanduser(str(output_dir)+"/"+source)
    
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
        
    output_dir = os.path.expanduser(output_dir)
    # test if link is direct download link
    
    # if link is not a route to a website download the content
    
    if "accessed" in link:
        new_link = link.split("accessed")[0]
        link = new_link
        
    print(link)
    if validators.url(link):      
        try:
            s = HTMLSession()
            
            r = s.get(link)
            if r.headers["Content-Type"] != "text/html" or r.headers["Content-Type"] == "application/x-Research-Info-Systems":
                # print(r.headers["Content-Type"])
                filename = ""
                try:
                    if "Content-Disposition" in r.headers:
                        filename = r.headers["Content-Disposition"].split("filename=")[1]
                except KeyError:
                    pass
                except IndexError:
                    pass
                if r.headers["Content-Type"] == "application/x-Research-Info-Systems":
                    filename = "RIS.txt"
                if filename != "":
                    # remove any unwanted characters
                    extension = filename.split(".")[-1]
                    # remove any unwanted characters from extension
                    extension = "."+"".join([i if i.isalnum() else "" for i in extension])
                    filename = "".join([i if i.isalnum() else "_" for i in filename]) + extension
                    with open(os.path.join(output_dir, filename), 'wb') as f:
                        f.write(r.content)
                        return [os.path.join(output_dir, filename)]
        except KeyError:
            pass
        except requests.exceptions.ConnectionError:
            pass
        except requests.exceptions.RequestException as e:
            print(f'RequestException: {e}')
            
        if "osf.io" in link:
            # print(link.split("/")[-2])
            zip_ = "https://files.osf.io/v1/resources/"+ link.split("/")[-2] +"/providers/osfstorage/?zip="
            fp = download_osf_file(zip_, output_dir, f'{link.split("/")[-2]}.zip')
            return [fp]
        elif "pangaea" in link:
            print(link, "PANGAEA")
            pangaea_formats = {
                "tab":"?format=textfile",
                "zip":"?format=zip"  
            }
            for f in pangaea_formats:
                # try:
                data_link = link+pangaea_formats[f]
                s = HTMLSession()
                r = s.get(data_link)
                with open(os.path.join(output_dir, f"dataset_pangaea.{f}"), 'wb') as f:
                    f.write(r.content)
                return [os.path.join(output_dir, f"dataset_pangaea.{f}")]
                # except:
                #     pass
        elif "dryad" in link:
            print(link, "DRYAD")
            doi = link.split("dx.doi.org")[1][1:]
            
            print(doi)
            
            url_doi = "doi%3A"+doi.replace("/", "%2F")
            
            dryad_download_api = f"https://datadryad.org/api/v2/datasets/{url_doi}/download"
            
            r = requests.get(dryad_download_api)
            with open(os.path.join(output_dir, f"dataset_dryad.zip"), 'wb') as f:
                f.write(r.content)

        elif "figshare" in link:
            print(link)
            s = HTMLSession()
            r = s.get(link)
            soup = BeautifulSoup(r.html.html, 'html.parser')
            figshare_link = ""
            for link in soup.find_all('a'):
                try:
                    if "ndownloader" in link.get('href'):
                        figshare_link = link.get('href')
                except TypeError:
                    pass
            if figshare_link != "":
                download_file(figshare_link, output_dir)
                return [os.path.join(output_dir, figshare_link.split("/")[-1])]
        elif "dataverse" in link:
            base_url = link.split("/dataset")
            print(base_url)
            
        else:
            try:
                s = HTMLSession()
                r = s.get(link)
                real_url = r.url
                domain = real_url.split('/')[2]
                soup = BeautifulSoup(r.html.html, 'html.parser')
                files = []
                for i in filetypes:
                    filename = ""
                    links = soup.find_all('a')
                    for link in links:
                        try:
                            if "."+i in link.get('href') or "."+i in link.get('title'):
                                l = link.get('href')
                                if "http" not in l:
                                    l = "https://"+domain + l
                                files.append(l)
                        except TypeError:
                            pass
                downloaded_files = []
                for i in files:
                    filename = i.split("/")[-1]
                    if "?" in filename:
                        filename = filename.split("?")[0]
                        
                    if filename != "":
                        if any([i in filename.lower() for i in filetypes]):
                            if filename not in downloaded_files:
                                # print("Downloading", filename)
                                download_file(i, output_dir)
                                downloaded_files.append(filename)
                    else:
                        pass
                        # print("Invalid filename", i)
                return downloaded_files
            except requests.exceptions.RequestException as e:
                print(f'RequestException: {e}')
                return []
    else:
        return []

def DownloadData(data_report, output_dir):
    print("")
    print("Attempting to download open data from the web.")
    print("Be advised that due to the nature of different websites, automatic download may not be possible.")
    print("Manual download may be required.")
    print("Please check the output directory for downloaded files.")
    print("")

    data_dirs = []
    number_of_files = []

    for idx, row in tqdm(data_report.iterrows()):
        data_amount = 0
        data_dir = ""
        # print(row)
        links = []
        for i in ["data_links_keywords", "data_links_web"]:
            try:
                links = links+row[i]
            except KeyError:
                pass
            except TypeError:
                pass
        # print(links)

        title = row["title"]
        # replace all special characters with underscores
        title = "".join([i if i.isalnum() else "_" for i in title])
        data_dir = Path(os.path.expanduser(str(output_dir)+"/"+title))

        data_dirs.append(str(data_dir))
        os.makedirs(data_dir, exist_ok=True)

        if len(links) > 0:
            for i in links:
                get_data_from_link(i, data_dir)
        # count downloaded files
        for root, dirs, files in os.walk(data_dir):
            for file in files:
                data_amount += 1

        number_of_files.append(data_amount)

    # for every dir in data_dirs check if it is empty and remove it
    new_data_dirs = []
    for i in data_dirs:
        if len(os.listdir(i)) == 0:
            os.rmdir(i)
            new_data_dirs.append("")
        else:
            new_data_dirs.append(i)
            
    delete_empty_folders(output_dir)

    data_report["data_dir"] = new_data_dirs
    data_report["number_of_files"] = number_of_files

    print("Download attempt complete.")

    return data_report

def delete_empty_folders(root):
   for dirpath, dirnames, filenames in os.walk(root, topdown=False):
      for dirname in dirnames:
         full_path = os.path.join(dirpath, dirname)
         if not os.listdir(full_path): 
            os.rmdir(full_path)


if __name__ == '__main__':
    doi = "10.1016/j.tpb.2012.08.002"
    doi = "10.3390/microorganisms10091765"
    
    links = find_data_web(doi)
    
    data = get_data_from_link(links[0], "~/Documents/pp")
    
    print(links)




# import torch 
# from transformers import AutoModelForCausalLM, AutoTokenizer, pipeline 


# def LoadModel():
#     # Check if GPU is available
#     device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
#     # check gpu memory ammount
#     vram = torch.cuda.get_device_properties(0).total_memory / 1024**3
#     device_ = "cuda"

#     if device.type != "cuda":
#         raise ValueError("This function is not supported on CPU")
#     if vram < 12:
#         device_ = "auto"
#         print("""

#     ECOOPEN WARNING:
#     Not enough GPU memory to run the model, switching to combined mode, which will be slower. Please use other methods (non AI) to analyze the text.

#     """)
        

#     torch.random.manual_seed(0) 
#     model = AutoModelForCausalLM.from_pretrained( 
#         "microsoft/Phi-3-mini-4k-instruct",  
#         device_map=device_,  
#         torch_dtype="auto",  
#         trust_remote_code=True,  
#     )
#     tokenizer = AutoTokenizer.from_pretrained("microsoft/Phi-3-mini-4k-instruct")
#     return model, tokenizer

# def get_inference(input_text, model):
#     model = model[0]
#     tokenizer = model[1]
#     # print(input_text)
#     answers = []
#     questions = [
#         f"Is the following text part of the scientific paper? {input_text}",
#         f"Does the following text contains information about supplementary data, data availability or a link to the data? {input_text}"
#     ]
#     for q in questions:
#         messages = [ 
#             {"role": "system", "content": "You are analyzing a possible snippet of a scientific paper. You are answering with yes and no."}, 
#             {"role": "user", "content": q},
#         ]
#         pipe = pipeline( 
#             "text-generation", 
#             model=model, 
#             tokenizer=tokenizer, 
#         )
#         generation_args = { 
#             "max_new_tokens": 500, 
#             "return_full_text": False, 
#             "temperature": 0.1, 
#             "do_sample": False, 
#         }
#         output = pipe(messages, **generation_args ) 
#         answers.append(output[0]['generated_text'])
        
#     return answers

# if __name__=="__main__":
#     # create_model()
#     print(get_inference("Data is available at the paper website. The link is www.sciom.hr/data/data.csv"))


from habanero import Crossref
import numpy as np
import re
from tqdm import tqdm
import pandas as pd
import itertools
from pathlib import Path
import sys
import requests
from bs4 import BeautifulSoup
import os
from urllib.parse import urlparse
# from urllib.error import 
import pathlib
from time import sleep
import pdfplumber
from pprint import pprint
from tqdm import tqdm
from datetime import datetime

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity

from EcoOpen.utils.keywords import keywords
from EcoOpen.data_download import *


def get_date_string(published):
    
    return "-".join([str(i) for i in published])
    

def FindPapers(query="", doi=[], number_of_papers=10, start_year=2010, 
               end_year=datetime.now().year+1, sort="relevance", order="desc"):

    number_of_papers = int(number_of_papers)
    cr = Crossref()
    if query != "":
        papers = cr.works(
            query=query,
            limit=number_of_papers,
            filter = {
                "type": "journal-article",
                'from-pub-date': str(start_year),
                'until-pub-date': str(end_year)})
    elif doi != []:
        cr = Crossref()
        papers = cr.works(ids=doi)
    else:
        raise ValueError("Please provide a keyword, doi, title or author")
    dataframe = {
        "doi":[],
        "title":[],
        "authors":[],
        "published":[],
        "link":[],
    }

    if type(papers) == dict:
        if doi != []:
            if type(papers["message"]) == dict:
                paper = papers["message"]
                try:
                    dataframe["doi"].append(paper["DOI"])
                except KeyError:
                    dataframe["doi"].append("")
                try:
                    dataframe["title"].append(paper["title"][0])
                except KeyError:
                    dataframe["title"].append("")
                try:
                    authors = ""
                    for author in paper["author"]:
                        authors += author["given"] + " " + author["family"] + ", "
                    authors = authors[:-2]
                
                    dataframe["authors"].append(authors)
                except KeyError:
                    dataframe["authors"].append("")
                try:
                    date = get_date_string(paper["published"]["date-parts"][0])
                    dataframe["published"].append(date)
                except KeyError:
                    dataframe["published"].append("")
                try:
                    dataframe["link"].append(paper["link"][0]["URL"])
                except KeyError:
                    dataframe["link"].append("")
        else:
            for paper in papers["message"]["items"]:
                # fill the dataframe. if the certain key is not present add empty string
                try:
                    dataframe["doi"].append(paper["DOI"])
                except KeyError:
                    dataframe["doi"].append("")
                try:
                    dataframe["title"].append(paper["title"][0])
                except KeyError:
                    dataframe["title"].append("")
                try:
                    authors = ""
                    for author in paper["author"]:
                        authors += author["given"] + " " + author["family"] + ", "
                    authors = authors[:-2]
                
                    dataframe["authors"].append(authors)
                except KeyError:
                    dataframe["authors"].append("")
                try:
                    date = get_date_string(paper["published"]["date-parts"][0])
                    dataframe["published"].append(date)
                except KeyError:
                    dataframe["published"].append("")
                try:
                    dataframe["link"].append(paper["link"][0]["URL"])
                except KeyError:
                    dataframe["link"].append("")
    elif type(papers) == list:
        for p in papers:
            # fill the dataframe. if the certain key is not present add empty string
            paper = p["message"]
            try:
                dataframe["doi"].append(paper["DOI"])
            except KeyError:
                dataframe["doi"].append("")
            try:
                dataframe["title"].append(paper["title"][0])
            except KeyError:
                dataframe["title"].append("")
            try:
                authors = ""
                for author in paper["author"]:
                    authors += author["given"] + " " + author["family"] + ", "
                authors = authors[:-2]
                dataframe["authors"].append(authors)
            except KeyError:
                dataframe["authors"].append("")
            try:
                date = get_date_string(paper["published"]["date-parts"][0])
                dataframe["published"].append(date)
            except KeyError:
                dataframe["published"].append("")
            try:
                dataframe["link"].append(paper["link"][0]["URL"])
            except KeyError:
                dataframe["title"].append("")
                
    df = pd.DataFrame(dataframe)
    o = False
    if order == "asc":
        o = True
    if sort == "published":
        df = df.sort_values(by="published", ascending=o, ignore_index=True)
    
    return df
    # return papers


def get_journal_info(journal_name):
    cr = Crossref()
    journal = cr.journals(query=journal_name)
    return journal


def get_paper_by_keyword(keyword):
    cr = Crossref()
    papers = cr.works(query=keyword)
    return papers


def get_papers(journal_list="available_journals.csv"):
    df = pd.read_csv(journal_list, index_col=0)
    df = df.dropna()
    df = df.reset_index()
    # print(df)

    for i in range(200, len(df)):
        # i = 101
        journal_title = df["Title"][i]
        expected_papers = df["Number of papers since 2010"][i]
        issn = df["ISSN"][i]
        issn = issn.replace("'", "")
        issn = issn.replace("[", "").replace("]", "")
        issn = issn.replace(" ", "").split(",")

        print(issn)

        noffsets=1
        if expected_papers/1000 >=1:
            noffsets = expected_papers//1000
        cr = Crossref()
        print(f"Searching for papers in {journal_title} since 01-01-2010")
        for iss in issn:
            jinfo = cr.journals(
                ids=issn,
                works=True,
                sort="published",
                order="asc",
                cursor="*",
                cursor_max=int(expected_papers),
                filter = {'from-pub-date': '2010-01-01'},
                progress_bar = True,
                limit=1000)

            print(type(jinfo))
            print(iss)

            if type(jinfo) == dict:
                items = jinfo["message"]["items"]
                if len(items) > 0:
                    break
            else:
                print(sum(len(z["message"]["items"]) for z in jinfo))

                items = [z["message"]["items"] for z in jinfo]
                items = list(itertools.chain.from_iterable(items))

        papers = {}

        keys = [
            "title",
            "published",
            "DOI",
            "type",
            #"abstract",
            "link",
            "is-referenced-by-count",
            "publisher",
            "author" # TODO: format
        ]
        for key in keys:
            papers[key] = []
        n=1
        for item in items:
            for key in keys:
                try:
                    value = item[key]
                    if key=="title":
                        value = value[0]
                    elif key in ("published", "issued"):
                        value = value["date-parts"][0]
                        if len(value) == 3:
                            value = f"{value[0]}-{value[1]}-{value[2]}"
                        elif len(value) == 2:
                            value = f"{value[0]}-{value[1]}"
                        else:
                            value = f"{value[0]}"
                    elif key == "link":
                        for v in value:
                            # print(v["URL"])
                            if "xml" not in v["URL"]:
                                value = v["URL"]
                            if "pdf" in v["URL"]:
                                value = v["URL"]
                                break
                            else:
                                value="no link"

                    if key == "author":
                        author_string=""
                        for a in value:
                            author_string+=a["given"] + " "
                            author_string+=a["family"]
                            if a["affiliation"] != []:
                                author_string+=" (" + str(a["affiliation"][0]["name"]) + "),"                    
                        value= author_string

                    papers[key].append(value)
                except KeyError:
                    # print(f"KeyError: {key}")
                    if key == "link":
                        key="URL"
                        try:
                            papers["link"].append(item[key])
                        except KeyError:
                            papers[key].append("")
                    else:
                        papers[key].append("")

            #print([item[k] for k in keys])

        jdf = pd.DataFrame(papers)
        jj = journal_title.replace(" ", "_")
        idx_ = i+1
        jdf.to_csv(f"data/papers/{idx_:03d}_{jj}.csv", sep=",", quoting=2)

    return "done!"

def custom_tokenizer(text):
    pattern = r"(?u)\b\w\w+\b[!]*"
    return re.findall(pattern, text) 

def gather_journal_info(list_of_journals, path_to_save=None):
    ISSNs = []
    titles = []
    query = []
    available = []
    dois = []
    number_of_papers_since_2010 = []
    journals = tqdm(list_of_journals, colour="green")

    for i in journals:
        jinfo = get_journal_info(i)

        if jinfo['message']["total-results"] > 0:

            # find item wit the most similar title
            # finding the right journal
            # in majority of cases the idx of the most similar journal title
            # will be 0, however sometimes API returns more popular journal
            # title as the first choice therefore an similarity checks between
            # query and result needs to be performed
            titles_ = [j["title"] for j in jinfo["message"]["items"]]
            qq = i
            vectorizer = TfidfVectorizer(
                tokenizer=custom_tokenizer, token_pattern=None)
            combined_list = titles_ + [qq]
            tfidf_matrix = vectorizer.fit_transform(combined_list)
            # print(tfidf_matrix)
            cosine_sim = cosine_similarity(
                tfidf_matrix[-1], tfidf_matrix[:-1]).flatten()

            idx = cosine_sim.argmax()
            # sanity check
            # if idx!=0:
            #     print(titles_)
            #     print(cosine_sim)
            ### idx points to the most similiar title
            
            available.append(1)
            ISSNs.append(jinfo['message']['items'][idx]['ISSN'])
            titles.append(jinfo['message']['items'][idx]['title'])
            
            npapers = 0
            dj = jinfo[
                "message"
                ]["items"][idx]["breakdowns"]["dois-by-issued-year"]
            for year in dj:
                if int(year[0]) >= 2010:
                    npapers += year[1]
            number_of_papers_since_2010.append(npapers)
            # print(
            #     i, "|",
            #     jinfo['message']["total-results"], "|",
            #     npapers, "|",
            #     titles[-1], "|",
            #     idx)
            res = jinfo['message']["total-results"]
            # Setting tqdm description is buggy
            # journals.set_description(
            #    f"{i} | {res} | {npapers} | {titles[-1]} | {idx}"
            #    )

        else:
            available.append(0)
            ISSNs.append("")
            titles.append("")
            number_of_papers_since_2010.append("")
            # print(
            #     i,
            #     jinfo['message']["total-results"]
            #     )
            # journals.set_description(f"{i} | no result found")
        query.append(i)

    df = pd.DataFrame({
        "Journal query": query,
        "Title": titles,
        "ISSN": ISSNs,
        "Available": available,
        "Number of papers since 2010": number_of_papers_since_2010
    })
    # drop duplicate rows
    # df.drop_duplicates()
    if path_to_save:
        df.to_csv(path_to_save)
    return df

def pdf_download(url, title, output_dir):
    
    # # print(pdf_url)
    
    # if pdf_url.startswith('//'):
    #     pdf_url = f"https:{pdf_url}"
    # elif pdf_url.startswith('/'):
    #     pdf_url = f"https:/{pdf_url}"
    
    # try:
    #     pdf_response = requests.get(pdf_url)
        
    #     if pdf_response.status_code == 200:
    #         file_name = f"{title.replace(' ', '_').replace('.', '_').replace('/', '_')}.pdf"
    #         file_path = f"{output_dir}/{file_name}"

    #         # Create the output directory if it doesn't exist
    #         os.makedirs(output_dir, exist_ok=True)
    #         # Save the content of the response as a PDF file
    #         with open(file_path, 'wb') as f:
    #             f.write(pdf_response.content)
            
    #         return True
    #     else:
    #         # print("Failed to download PDF")
    #         return None
    # except requests.exceptions.ConnectionError:
    #     return None
    # except:
    
    #     return None
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
    }
    file_name = f"{title.replace(' ', '_').replace('.', '_').replace('/', '_')}.pdf"
    # print(url)
    # print(output_dir)
    # print(file_name)
    try:
        # Send a GET request to the URL
        response = requests.get(url,  headers=headers, stream=True)
        
        # Check if the request was successful
        if response.status_code == 200:
            # Create the output directory if it doesn't exist
            os.makedirs(output_dir, exist_ok=True)
            
            # Define the file path
            file_path = os.path.join(output_dir, file_name)
            
            # Save the content of the response as a file
            with open(file_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=8192):
                    file.write(chunk)
            
            # print(f"File downloaded successfully and saved as {file_path}")
            return True
        else:
            # print(f"Failed to download file. Status code: {response.status_code}")
            return None
    except requests.exceptions.RequestException as e:
        # print(f"An error occurred: {e}")
        return None


def other_sources_download(doi, title, output_dir):
    output_dir = Path(os.path.expanduser(output_dir))
    output_dir = output_dir.absolute()
    # print(output_dir)

    if not os.path.exists(output_dir):
        os.system(f"mkdir {output_dir}")
        
    site_url = "https://sci-hub.se/"
    
    download_link = site_url+doi
    # Send a GET request to the Sci-Hub page
    response = requests.get(download_link, timeout=10)

    # Check if the request was successful
    if response.status_code == 200:
        # Parse the HTML content using BeautifulSoup
        soup = BeautifulSoup(response.content, 'html.parser')
        
        # Find the iframe or embed tag containing the PDF
        iframe = soup.find('iframe')
        if iframe:
            pdf_url = iframe['src']
        else:
            embed = soup.find('embed')
            if embed:
                pdf_url = embed['src']
            else:
                # print("PDF URL not found")
                return None
        # Handle relative URLs
        if pdf_url.startswith("/downloads"):
            pdf_url = f"https://sci-hub.se{pdf_url}"
        if pdf_url.startswith('//'):
            pdf_url = f"https:{pdf_url}"
        elif pdf_url.startswith('/'):
            pdf_url = f"https:/{pdf_url}"
            
        # Send a GET request to the extracted PDF URL
        # print(pdf_url)
        try:
            pdf_response = requests.get(pdf_url, timeout=3)

            # Check if the request was successful
            if pdf_response.status_code == 200:
                
                # remove non alphanumeric characters from the title
                title = re.sub(r'\W+', '', title)
                
                file_name = f"{title.replace(' ', '_').replace('.', '_').replace('/', '_')}.pdf"
                file_path = f"{output_dir}/{file_name}"

                # Create the output directory if it doesn't exist
                os.makedirs(output_dir, exist_ok=True)
                # Save the content of the response as a PDF file
                with open(file_path, 'wb') as f:
                    f.write(pdf_response.content)
                # print(f"Downloaded {file_name}")
                return True
            else:
                pass
        except requests.exceptions.ConnectionError:
            print("Connection error")
            print(pdf_url)
    else:
        pass


def download_paper(paper, output_dir, other_sources=False):
    output_dir = Path(os.path.expanduser(output_dir))
    output_dir = output_dir.absolute()
    # print(output_dir)

    link = paper["link"]
    title = paper["title"]
    doi = paper["doi"]

    # print(doi, title, link)
    
    download_link = ""

    if "pdf" not in link and link != "":
        download_link = link
        download_link = find_pdf(download_link)
    else:
        download_link = link

    # print(download_link, "!!!", title)

    if not os.path.exists(output_dir):
        try:
            os.mkdir(output_dir)
        except FileNotFoundError:
            os.makedirs(output_dir, exist_ok=True)
    existing_papers = list(Path(f"{output_dir}").glob("**/*.pdf"))
    
    new_path = ""
    # print(download_link, "download link")
    if download_link == "":
        # print("PDF link was not found in the paper metadata or on the webpage")
        # print("Switching to Sci-Hub")
        if other_sources == True:
            try:
                r = other_sources_download(doi, title, output_dir)
                if r != None:
                    new_path = f'{output_dir}/{title.replace(" ", "_").replace(".","_").replace("/", "_")}.pdf'
            except ConnectionError:
                # print("Download failed")
                new_path = ""
        else:
            # print("No PDF link found")
            new_path
    elif "pdf" in download_link:
        # print("Downloading PDF from the provided link", download_link)
        r = pdf_download(download_link, title, output_dir)
        # print(r, "vanilla")
        if r != None:
            new_path = f'{output_dir}/{title.replace(" ", "_").replace(".","_").replace("/", "_")}.pdf'
    
    # print(new_path, "path")
    if new_path == "":
        return None
    elif Path(new_path).exists():
        # get path to the downloaded file that was not present before
        # print("Download successful", title)
        return new_path
    else:
        # print("Download failed")
        return None

def find_pdf(url):
    # Requests URL and get response object
    try:
        response = requests.get(url, timeout=3)
        # print(response.url)
        # Parse text obtained
        try:
            soup = BeautifulSoup(response.text, 'html.parser')
        except:
            soup = BeautifulSoup(response.text, "lxml")
        # Find all hyperlinks present on webpage
        links = soup.find_all('a')
        i = 0
        # From all links check for pdf link and
        # if present download file
        the_link = ""
        for link in links:
            # print(link)
            if ('pdf' in link.get('href', [])):
                i += 1
                # print(link.get('href', []))
                the_link = link.get('href', [])
                break

        # print(the_link, "The link")
        if the_link == "":
            return ""
        elif "http" not in the_link:
            return "http://"+urlparse(response.url).netloc+the_link
        else:
            return the_link
    except requests.exceptions.ConnectionError:
        return ""

def find_das(sentences):
    """Find data availability sentences in the text"""
    das_keywords = keywords["data_availability"]
    das_sentences = [sentence for sentence in sentences if any(kw.lower() in sentence.lower() for kw in das_keywords)]
    return das_sentences

def find_keywords(sentences):
    """find keywords in sentences"""
    detected = []
    kw = keywords.copy()
    kw.pop("data_availability")
    for k in kw.keys():
        kk = kw[k]
        for sentence in sentences:
            if any(kw.lower() in sentence.lower() for kw in kk):
                detected.append(sentence)
    return detected

def find_dataKW(path):
    """Find data keywords in the text"""
    raw = ReadPDF(path)
    sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', raw)
    das_sentences = find_das(sentences)
    keyword_sentences = find_keywords(sentences)
    
    
    # detect links in the sentences
    data_links = []
    for sentence in keyword_sentences:
        link = re.findall(r'(https?://\S+)', sentence)
        if link != []:
            data_links.append(link)
    
    # clean the links
    dl = []
    for i in data_links:
        for j in i:
            # split merged links
            if len(j.split("http")) > 2:
                for k in j.split("http"):
                    if k != "":
                        if ("http"+k) not in dl:
                            dl.append("http"+k)
            else:
                if j not in dl:
                    dl.append(j)
    # clean special non standard url characters
    dl = [i.replace(")", "").replace("]", "").replace("}", "") for i in dl]
    dl = [i.replace("(", "").replace("[", "").replace("{", "") for i in dl]
    dl = [i.replace(";", "").replace(",", "") for i in dl]
    return das_sentences, dl

def DownloadPapers(papers, output_dir, other_sources=False):
    print("Downloading papers")
    download_results = {"doi":[], "path": [], "result":[]}
    
    pbar = tqdm(total=len(papers))
    # with tqdm(total=len(papers)) as pbar:
    for idx, paper in tqdm(papers.iterrows()):
    
        paper = papers.iloc[idx]
        result = download_paper(paper, output_dir, other_sources)
        doi = paper["doi"]
        # pbar.set_description(f"Downloading {doi}")
        
        # print(type(result), "result")
        if result != None:
            download_results["doi"].append(doi)
            download_results["path"].append(result)
            download_results["result"].append("Success")
        else:
            download_results["doi"].append(doi)
            download_results["path"].append("Article publicly unavailable")
            download_results["result"].append("Article unavailable")
            
        pbar.update(1)


    # merge the results by doi
    download_results =  pd.DataFrame(download_results)

    download_results = pd.merge(papers, download_results, on="doi")

    return download_results


def ReadPDF(filepath):
    # parse PDF using 
    raw = ""
    try:
        pdf = pdfplumber.open(filepath)
        for page in pdf.pages:
            raw += page.extract_text()
        pdf.close()
    except:
        pass
    return raw.replace("\n", "")

def ReadPDFs(filepaths):
    # if filepaths is a dataframe extract filepaths
    if type(filepaths) == pd.core.frame.DataFrame:
        filepaths = filepaths["path"].tolist()
    raws = []
    for file in filepaths:
        if type(file) == pathlib.PosixPath:
            file = str(file)
        raw = ReadPDF(file)
        raws.append(raw)
    return raws


def FindOpenData(files, method="keywords"):
    """Prototype function to find open data in scientific papers"""

    # if "path" not in files.columns:
    #     print("Dataframe does not contain path column")
    #     print("Downloading papers")
    #     if output_dir == "":
    #         raise ValueError(
    #             "Please provide output directory using output_dir argument")
    #     files = download_papers(files, output_dir, scihub=False)
    print(f"Finding open data using '{method}' method:")
    data_links = []

    if method == "keywords":
        for i in tqdm(files["path"].tolist()):
            try:
                das, dl = find_dataKW(i)
                data_links.append(dl)
            except FileNotFoundError:
                data_links.append([])
    elif method == "web":
        for i in tqdm(files["doi"].tolist()):
            dl = find_data_web(i)
            data_links.append(dl)
    else:
        raise ValueError("Method not recognized, use keywords or web!")

    # print(len(files))
    # print(len(data_links))
    files["data_links"+f"_{method}"] = data_links

    return files

def analyze_reference_document(path):
    """
    Analyze reference document for data availability
    Used to analyze references in pdf or other text documents
    """

    # reference document analysis routine
    result = {
        "doi":[],
        "title":[],
        "authors":[],
        "original_data_present": [],
        "data_links": []
    }
    return result


if __name__ == "__main__":
    papers = FindPapers("Hackenberger")



# from EcoOpen.inference import get_inference
# from EcoOpen.core import ReadPDF
# from tqdm import tqdm
# import re
# from pprint import pprint
# import os
# import torch
# from transformers import BertTokenizer, BertForSequenceClassification

# def AnalyzePDF_AI(filepath, model):
#     raw = ReadPDF(filepath)
#     # split text into sentences
#     sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', raw)
#     # pprint(sentences)

#     # split raw into paragraphs made from 5 sentences
#     paragraphs = []
#     for i in range(0, len(sentences), 3):
#         paragraph = " ".join(sentences[i:i+5])
#         paragraphs.append(paragraph)

#     # get inference
#     inference = []
#     for i in tqdm(paragraphs):
#         inf = get_inference(i, model)
#         # print("Sentece", i)
#         # print("Inference", inf)
#         inference.append([i, inf])

#     inference = inference_cleanup(inference)
    
#     return inference

# def inference_cleanup(inference):
#     new_inference = []
#     for i in inference:
#         i_=[i[0]]
#         if "yes" in i[1][0].lower():
#             i_.append("yes")
#         elif "no" in i[1][0].lower():
#             i_.append("no")

#         if "yes" in i[1][1].lower():
#             i_.append("yes")
#         elif "no" in i[1][1].lower():
#             i_.append("no")

#         new_inference.append(i_)
#     return new_inference


# def find_dataAI(path):
#     inference = AnalyzePDF_AI(path, model)
#     indices = find_supplementary_data_sentence_AI(inference)
#     data_links = []
#     dataframe = {
#         "data_links": [],
#         "inference": []
#     }
#     if indices != []:
#         for i in indices:
#             # detect http links
#             link = re.findall(r'(https?://\S+)', inference[i][0])
#             if link != []:
#                 data_links.append(link)

#         dataframe["data_links"] = data_links
#         dataframe["inference"] = inference[i]

#         return dataframe

#     else:
#         print("No supplementary data found or data reference found in this paper")
#         return None

# def find_supplementary_data_sentence_AI(inference):
#     indices = []
#     for idx, i in enumerate(inference):
#         if "yes" in i[2].lower():
#             indices.append(idx)
#     return indices


import pandas as pd
import os

cwd = os.getcwd()
cwd = cwd.replace(
    "/utils", ""
    )

journals = pd.read_csv(
    cwd+"/data/List_ecoevo_journals_latest.csv", index_col=0)

ID = journals.index.values
Journal = journals["Journal query"].values

data = list(Journal)
data = [i.replace("&", "") for i in data]
data = [i.replace(" and ", " ") for i in data]

journals = pd.DataFrame(columns=["Journal"], data=data)
journals.sort_values(by="Journal", inplace=True, ignore_index=True)
journals = journals["Journal"]



from setuptools import setup, find_packages

setup(
    name="EcoOpen",
    version="0.0.49",
    packages=find_packages(),
    install_requires=[
        "habanero",
        "numpy",
        "pandas",
        "scipy",
        "pdfplumber",
        "beautifulsoup4",
        "scikit-learn",
        "jupyterlab",
        "lxml",
        "requests-html",
        "lxml-html-clean",
        "validators",
        "pyDataverse"
    ],
    # entry_points={
    #     "console_scripts": [
    #         "EcoOpen=EcoOpen.core:main"
    #     ]
    # }
    author="Sciom d.o.o.",
    author_email="domagoj@sciom.hr",
    description="A tool for gathering information about data and data in scientific journals.",
    license="MIT",
    keywords="scientific journals, open access, data_mining",
    url="https://sciom.hr"
)



from habanero import Crossref
import pandas as pd

doi = ["10.1016/j.ecolmodel.2011.10.027", "10.1016/j.ecolmodel.2013.06.014"]

cr = Crossref()

papers = cr.works(ids=doi)
for paper in papers["message"]["items"]:
    # fill the dataframe. if the certain key is not present add empty string
    dataframe = {
        "doi":[],
        "title":[],
        "authors":[],
        "published":[],
        "link":[],
    }   
    
    try:
        dataframe["doi"].append(paper["DOI"])
    except KeyError:
        dataframe["doi"].append("")
    try:
        dataframe["title"].append(paper["title"][0])
    except KeyError:
        dataframe["title"].append("")
    try:
        authors = ""
        for author in paper["author"]:
            authors += author["given"] + " " + author["family"] + ", "
        authors = authors[:-2]
    
        dataframe["authors"].append(authors)
    except KeyError:
        dataframe["authors"].append("")
    try:
        dataframe["published"].append(paper["published"]["date-parts"][0])
    except KeyError:
        dataframe["published"].append("")
    try:
        dataframe["link"].append(paper["link"][0]["URL"])
    except KeyError:
        dataframe["link"].append("")
        
        
df = pd.DataFrame(dataframe)
